// Generated by CoffeeScript 1.3.1

/*

  Harvey coinManager — Copyright (c) 2012 Joschka Kintscher
*/


(function() {
  var Coin, _mediaQueryList,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  this.Harvey = (function() {

    Harvey.name = 'Harvey';

    function Harvey() {}

    Harvey.coins = {};

    Harvey.queries = [];

    Harvey.attach = function(mediaQuery, callbacks) {
      var coin;
      if (!this.coins.hasOwnProperty(mediaQuery)) {
        this.coins[mediaQuery] = [];
        this._add_css_for(mediaQuery);
      }
      coin = new Coin(mediaQuery, callbacks != null ? callbacks.setup : void 0, callbacks != null ? callbacks.on : void 0, callbacks != null ? callbacks.off : void 0);
      this.coins[mediaQuery].push(coin);
      if (__indexOf.call(this.queries, mediaQuery) < 0) {
        this._watch_query(mediaQuery);
      }
      if (this._window_matchmedia(mediaQuery).matches) {
        this._update_coins([coin], true);
      }
      return coin;
    };

    Harvey.detach = function(coin) {
      var c, i, _i, _len, _ref, _results;
      _ref = this.coins[coin.condition];
      _results = [];
      for (i = _i = 0, _len = _ref.length; _i < _len; i = ++_i) {
        c = _ref[i];
        if (coin === c) {
          _results.push(this.coins[c.condition][i] = void 0);
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    Harvey._watch_query = function(mediaQuery) {
      var _this = this;
      this.queries.push(mediaQuery);
      return this._window_matchmedia(mediaQuery).addListener(function(mql) {
        return _this._update_coins(_this.coins[mediaQuery], mql.matches);
      });
    };

    Harvey._update_coins = function(coins, active) {
      var coin, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = coins.length; _i < _len; _i++) {
        coin = coins[_i];
        if (active) {
          _results.push(coin.activate());
        } else {
          _results.push(coin.deactivate());
        }
      }
      return _results;
    };

    /*
    
        BEWARE: You're at the edge of the map, mate. Here there be monsters!
    
        ------------------------------------------------------------------------------------
    
        Private methods to fix and polyfill the matchMedia interface for several engines
    
        * Inspired by Nicholas C. Zakas' article on the different problems with matchMedia
          http://www.nczonline.net/blog/2012/01/19/css-media-queries-in-javascript-part-2/
    
        * Implementing a modified coffeescript version of Paul Irish's matchMedia.js polyfill
          https://github.com/paulirish/matchMedia.js
    */


    /*
        [FIX] for Firefox/Gecko browsers that lose reference to the
        MediaQueryList object unless it's being stored for runtime
    */


    Harvey._mediaList = {};

    Harvey._window_matchmedia = function(mediaQuery) {
      if (window.matchMedia) {
        if (!(mediaQuery in this._mediaList)) {
          this._mediaList[mediaQuery] = window.matchMedia(mediaQuery);
        }
        return this._mediaList[mediaQuery];
      }
      /*
            [POLYFILL] for all browsers that don't support matchMedia() at all (CSS media query support is mandatory though)
      */

      if (!this._listening) {
        this._listen();
      }
      if (!(mediaQuery in this._mediaList)) {
        this._mediaList[mediaQuery] = new _mediaQueryList(mediaQuery);
      }
      return this._mediaList[mediaQuery];
    };

    Harvey._listen = function() {
      var evt,
        _this = this;
      evt = window.addEventListener || window.attachEvent;
      evt('resize', function() {
        var mediaList, mediaQuery, _ref, _results;
        _ref = _this._mediaList;
        _results = [];
        for (mediaQuery in _ref) {
          mediaList = _ref[mediaQuery];
          _results.push(mediaList._process());
        }
        return _results;
      });
      evt('orientationChange', function() {
        var mediaList, mediaQuery, _ref, _results;
        _ref = _this._mediaList;
        _results = [];
        for (mediaQuery in _ref) {
          mediaList = _ref[mediaQuery];
          _results.push(mediaList._process());
        }
        return _results;
      });
      return this._listening = true;
    };

    /*
        [FIX] for Webkit engines that only trigger the MediaQueryListListener
        when there is at least one CSS selector for the respective media query
    */


    Harvey._add_css_for = function(mediaQuery) {
      if (!this.style) {
        this.style = document.createElement('style');
        document.getElementsByTagName('head')[0].appendChild(this.style);
      }
      return this.style.appendChild(document.createTextNode("@media " + mediaQuery + " {.harvey-test{}}"));
    };

    return Harvey;

  })();

  Coin = (function() {

    Coin.name = 'Coin';

    Coin.prototype.active = false;

    Coin.prototype.is_setup = false;

    function Coin(condition, setup, on, off) {
      this.condition = condition;
      this.setup = setup;
      this.on = on;
      this.off = off;
    }

    Coin.prototype.activate = function() {
      if (this.active) {
        return;
      }
      if (!this.is_setup) {
        if (typeof this.setup === "function") {
          this.setup();
        }
        this.is_setup = true;
      }
      if (typeof this.on === "function") {
        this.on();
      }
      return this.active = true;
    };

    Coin.prototype.deactivate = function() {
      if (!this.active) {
        return;
      }
      if (typeof this.off === "function") {
        this.off();
      }
      return this.active = false;
    };

    return Coin;

  })();

  /*
    [FIX]/mimic of the matchMedia interface modified to work as a drop-in replacement for Harvey
  */


  _mediaQueryList = (function() {

    _mediaQueryList.name = '_mediaQueryList';

    function _mediaQueryList(media) {
      this.media = media;
      this._callbacks = [];
      this.matches = this._matches();
    }

    _mediaQueryList.prototype.addListener = function(listener) {
      this._callbacks.push(listener);
      return;
    };

    _mediaQueryList.prototype._process = function() {
      var callback, current, _i, _len, _ref, _results;
      current = this._matches();
      if (this.matches === current) {
        return;
      }
      this.matches = current;
      _ref = this._callbacks;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        callback = _ref[_i];
        _results.push(callback(this));
      }
      return _results;
    };

    _mediaQueryList.prototype._matches = function() {
      if (!this._tester) {
        this._get_tester();
      }
      this._tester.innerHTML = '&shy;<style media="' + this.media + '">#harvey-mq-test{width:42px;}</style>';
      this._tester.removeChild(this._tester.firstChild);
      return this._tester.offsetWidth === 42;
    };

    _mediaQueryList.prototype._get_tester = function() {
      this._tester = document.getElementById('harvey-mq-test');
      if (!this._tester) {
        return this._build_tester();
      }
    };

    _mediaQueryList.prototype._build_tester = function() {
      this._tester = document.createElement('div');
      this._tester.id = 'harvey-mq-test';
      this._tester.style.cssText = 'position:absolute;top:-100em';
      return document.body.insertBefore(this._tester, document.body.firstChild);
    };

    return _mediaQueryList;

  })();

}).call(this);
